class WktimeController < ApplicationController
unloadable

before_filter :require_login
before_filter :check_perm_and_redirect, :only => [:edit, :update, :destroy]

  def index
    cond = ARCondition.new
    retrieve_date_range
	@from = getMonday(@from)
	@to = getSunday(@to)
	
	cond << ['spent_on BETWEEN ? AND ?', @from, @to]
	if !User.current.admin?
		cond << ['user_id = ?', User.current.id]
	end

    respond_to do |format|
      format.html {
        # Paginate results
		set_user_projects
		ids = nil
		@user_ids.values.each_with_index do|id, i|
			if i == 0
				ids =  id.to_s
			else
				ids +=  ',' + id.to_s
			end
		end		
		ids = User.current.id.to_s if ids.nil?
		sqlStr = "select user_id, monday as spent_on, hours from " +
			" (select adddate(t.spent_on,weekday(t.spent_on)*-1) as monday, "
		
		# postgre doesn't have the weekday function
		if ActiveRecord::Base.connection.adapter_name == 'PostgreSQL'
			sqlStr = "select user_id, monday as spent_on, hours from " +
				" (select t.spent_on - cast(extract(dow from t.spent_on) as integer) + 1 as monday, "
		elsif ActiveRecord::Base.connection.adapter_name == 'SQLite'
			sqlStr = "select user_id, monday as spent_on, hours from " +
				" (select date(spent_on, '-' || strftime('%w', spent_on, '-1 days') || ' days') as monday, "
		end
		sqlStr += " t.user_id, sum(t.hours) as hours from time_entries t, users u" +
			" where u.id = t.user_id and u.id in (" + 
			ids +") and t.spent_on between '" + 
			@from.to_s() + "' and '" + @to.to_s() + "' group by monday, user_id order by monday desc, user_id) as v1"

		 
		result = TimeEntry.find_by_sql("select count(*) as id from (" + sqlStr + ") as v2")
		@entry_count = result[0].id
        @entry_pages = Paginator.new self, @entry_count, per_page_option, params['page']
			
		@entries = TimeEntry.find_by_sql(sqlStr + 
			" LIMIT " + @entry_pages.items_per_page.to_s +
			" OFFSET " + @entry_pages.current.offset.to_s)

        @total_hours = TimeEntry.visible.sum(:hours, :include => [:user], :conditions => cond.conditions).to_f

        render :layout => !request.xhr?
      }
    end

  end

  def edit
	cond = getCondition
	
	respond_to do |format|
		format.html {
			@entries = TimeEntry.find(:all, :conditions => cond.conditions,
				:order => 'project_id, issue_id, activity_id, spent_on')
			set_user_projects
			render :layout => !request.xhr?
		} 
		format.api  {
		}
	end
  end

  # called when save is clicked on the page
  def update
  	respond_to do |format|
		format.html {
			gatherEntries
			errorMsg = nil
			TimeEntry.transaction do
				begin
				errorMsg = l(:error_wktime_save_nothing) if @entries.blank?
				@entries.each do |entry|
						if entry.hours.blank?
							# delete the time_entry
							# if the hours is empty but id is valid
							# entry.destroy() unless ids[i].blank?
							if !entry.id.blank?
								if !entry.destroy()
									errorMsg = entry.errors.full_messages.join('\n')
									break
								end
							end
						else
							#if id is there it should be update otherwise create
							if !entry.save()
								errorMsg = entry.errors.full_messages.join('\n')
								break
							end					
						end
					end
				rescue Exception => e
					errorMsg = e.message
				end
				if errorMsg.nil?
					flash[:notice] = l(:notice_successful_update)
					redirect_back_or_default :action => 'index'
				else
					flash[:error] = l(:error_wktime_save_failed, errorMsg)
					render_edit
					raise ActiveRecord::Rollback
				end
			end
		} 
		format.api  {
		}
	end  
  end
	
	def deleterow
		if check_permission
			ids = params['ids']
			TimeEntry.delete(ids)	
			respond_to do |format|
				format.text  { 
					render :text => 'OK' 
				}
			end
		else
			respond_to do |format|
				format.text  { 
					render :text => 'FAILED' 
				}
			end
		end
	end

	def destroy
		cond = getCondition
	
		respond_to do |format|
			format.html {
				TimeEntry.delete_all(cond.conditions)
				flash[:notice] = l(:notice_successful_delete)
				redirect_back_or_default :action => 'index'
			} 
			format.api  {
			}
		end
	end
	
	def new
		set_user_projects
		render :action => 'new'
	end
	
  def getissues
	issues = Issue.find_all_by_project_id(params[:project_id])
	user = User.find(params[:user_id])
	
	issStr =""
	issues.each do |issue|
		issStr << issue.id.to_s() + ',' + issue.subject + "\n" if issue.visible?(user)
	end
	respond_to do |format|
		format.text  { render :text => issStr }
	end
  end
  
	def getactivities
		project = Project.find(params[:project_id])
		actStr =""
		project.activities.each do |a|
			actStr << a.id.to_s() + ',' + a.name + "\n"
		end
	
		respond_to do |format|
			format.text  { render :text => actStr }
		end
	end
	
private
	def getCondition
    cond = ARCondition.new
	@mon = params[:mon].to_s.to_date unless params[:mon].blank?
	
	# if user has changed the monday
	@mon = getMonday(@mon)
	
	cond << ['spent_on BETWEEN ? AND ? AND user_id = ?', @mon, @mon + 6, params[:user_id]]
	end
	#change the date to a monday
	def getMonday(date)
		unless date.nil?
			day1_diff = date.cwday%7 - 1
			date -= day1_diff if day1_diff != 0
		end
		date
	end

	#change the date to a sunday
	def getSunday(date)
		unless date.nil?
			day7_diff = 7 - date.cwday%7
			date += day7_diff if day7_diff != 7
		end
		date
	end
	
  def gatherEntries
		entryHash = params[:time_entry]
		@entries ||= Array.new
		@mon = params[:mon].to_s.to_date unless params[:mon].blank?

		entrylookupHash = Hash.new
		unless entryHash.nil?
			entryHash.each_with_index do |entry, i|
				unless entry['project_id'].blank?
					hours = params['hours' + (i+1).to_s()]
					ids = params['ids' + (i+1).to_s()]
					hours.each_with_index do |hour, i|
						if !ids[i].blank? || !hour.blank?
							timeEntry = nil
							uniqueKey = entry['project_id']+entry['issue_id']+
								entry['activity_id']+ (@mon + i).to_s
							if  entrylookupHash.key?(uniqueKey) && ids[i].blank?
								# For new entries, if the entry is already there, add the hours to it
								timeEntry = entrylookupHash[uniqueKey]
								timeEntry.hours += hour.to_f
							else
								if ids[i].blank?
									timeEntry = TimeEntry.new
								else
									# find it from the db so it can be updated
									timeEntry = TimeEntry.find(ids[i])
								end
								entrylookupHash[uniqueKey] = timeEntry
								
								timeEntry.attributes = entry
								# since project_id and user_id is protected
								timeEntry.project_id = entry['project_id']
								timeEntry.user_id = params[:user_id]
								timeEntry.spent_on = @mon + i
								if hour.blank?
									timeEntry.hours = nil
								else
									timeEntry.hours = hour.to_f
								end
							end							
							# add the entries to the array, so it can be used 
							# on the edit page if an error occurs
							@entries << timeEntry				
						end				
					end
				end
			end
		end
  end

	def render_edit
		set_user_projects
		render :action => 'edit', :user_id => params[:user_id], :mon => @mon
	end
	
  def check_perm_and_redirect
    unless check_permission
      render_403
      return false
    end
  end
  
  def check_permission
	set_user_projects
    return @user_ids.value?(params[:user_id].to_i)
  end
  
  # Retrieves the date range based on predefined ranges or specific from/to param dates
  def retrieve_date_range
    @free_period = false
    @from, @to = nil, nil

    if params[:period_type] == '1' || (params[:period_type].nil? && !params[:period].nil?)
      case params[:period].to_s
      when 'today'
        @from = @to = Date.today
      when 'yesterday'
        @from = @to = Date.today - 1
      when 'current_week'
        @from = Date.today - (Date.today.cwday - 1)%7
        @to = @from + 6
      when 'last_week'
        @from = Date.today - 7 - (Date.today.cwday - 1)%7
        @to = @from + 6
      when '7_days'
        @from = Date.today - 7
        @to = Date.today
      when 'current_month'
        @from = Date.civil(Date.today.year, Date.today.month, 1)
        @to = (@from >> 1) - 1
      when 'last_month'
        @from = Date.civil(Date.today.year, Date.today.month, 1) << 1
        @to = (@from >> 1) - 1
      when '30_days'
        @from = Date.today - 30
        @to = Date.today
      when 'current_year'
        @from = Date.civil(Date.today.year, 1, 1)
        @to = Date.civil(Date.today.year, 12, 31)
      end
    elsif params[:period_type] == '2' || (params[:period_type].nil? && (!params[:from].nil? || !params[:to].nil?))
      begin; @from = params[:from].to_s.to_date unless params[:from].blank?; rescue; end
      begin; @to = params[:to].to_s.to_date unless params[:to].blank?; rescue; end
      @free_period = true
    else
      # default
    end
    
    @from, @to = @to, @from if @from && @to && @from > @to
    @from ||= (TimeEntry.earilest_date_for_project(@project) || Date.today)
    @to   ||= (TimeEntry.latest_date_for_project(@project) || Date.today)
  end  

	def set_user_projects
		@projects = Array.new
		proj_members = Array.new
		@user_ids = Hash.new
		unless params[:user_id].blank?
			@user ||= User.find(params[:user_id])
			@user_ids[@user.login] = @user.id if User.current.id == @user.id
		end
		allProjects = Project.find(:all, :order => 'name')
		allProjects.each do |p|
			#get the list of allowable projects.
			if !@user.nil? && @user.allowed_to?(:log_time, p)
				@projects << p
			end
			# if the user can manage other isers then add them in
			if User.current.allowed_to?(:manage_members, p)
				p.members.each do |m|			
					@user_ids[m.user.login] = m.user.id unless @user_ids.key?(m.user.login)
				end
			end
			@user_ids[User.current.login] = User.current.id if @user_ids.empty?
		end
		@sorted_user_ids = @user_ids.sort_by{|login, id| login}
	end

end